%bcond_without openmpi
%bcond_without mpich
%bcond_without optimized_blas

%global blaslib flexiblas
%if %{with optimized_blas}
%global blasflags -DBLA_VENDOR=FlexiBLAS
%else
%global blasflags -DBLA_VENDOR=Generic
%endif

Summary: A subset of LAPACK routines redesigned for heterogeneous computing
Name: scalapack
Version: 2.2.0
Release: 2%{?dist}
License: Public Domain
URL: http://www.netlib.org/scalapack/
Source0: https://github.com/Reference-ScaLAPACK/scalapack/archive/v%{version}.tar.gz
Patch3000: scalapack-2.2-fix-version.patch
Patch3001: scalapack-2.2-set-CMAKE_POSITION_INDEPENDENT_CODE.patch
Patch3002: scalapack-2.2.0-fix57.patch


BuildRequires: cmake
%if %{with optimized_blas}
BuildRequires: %{blaslib}-devel
%else
BuildRequires: lapack-devel blas-devel
%endif
BuildRequires: gcc-gfortran glibc-devel
%if %{with mpich}
BuildRequires: mpich-devel-static
%endif
%if %{with openmpi}
BuildRequires: openmpi-devel
%endif

%description
ScaLAPACK is a library of high-performance linear algebra routines for parallel distributed 
memory machines. ScaLAPACK solves dense and banded linear systems, least squares problems, 
eigenvalue problems, and singular value problems. 

%package common
Summary: Common files for scalapack
Provides: blacs-common = %{version}-%{release}

%description common
ScaLAPACK is a library of high-performance linear algebra routines for parallel distributed 
memory machines. ScaLAPACK solves dense and banded linear systems, least squares problems, 
eigenvalue problems, and singular value problems. 

This package contains common files which are not specific to any MPI implementation.

%if %{with mpich}
%package mpich
Summary: ScaLAPACK libraries compiled against mpich
Requires: %{name}-common = %{version}-%{release}
Requires: mpich
Provides: %{name}-mpich2 = %{version}-%{release}
Provides: %{name}-lam = %{version}-%{release}
Provides: blacs-mpich = %{version}-%{release}

%description mpich
ScaLAPACK is a library of high-performance linear algebra routines for parallel distributed 
memory machines. ScaLAPACK solves dense and banded linear systems, least squares problems, 
eigenvalue problems, and singular value problems. 

This package provides ScaLAPACK libraries compiled with mpich.

%package mpich-devel
Summary: Development libraries for ScaLAPACK (mpich)
Requires: %{name}-mpich = %{version}-%{release}
Requires: mpich-devel
Provides: %{name}-lam-devel = %{version}-%{release}
Provides: %{name}-mpich2-devel = %{version}-%{release}
Provides: blacs-mpich-devel = %{version}-%{release}

%description mpich-devel
This package provides development libraries for ScaLAPACK, compiled against mpich.

%package mpich-static
Summary: Static libraries for ScaLAPACK (mpich)
Requires: %{name}-mpich-devel = %{version}-%{release}
Provides: %{name}-lam-static = %{version}-%{release}
Provides: %{name}-mpich2-static = %{version}-%{release}
Provides: blacs-mpich-static = %{version}-%{release}

%description mpich-static
This package provides static libraries for ScaLAPACK, compiled against mpich.

%endif

%if %{with openmpi}
%package openmpi
Summary: ScaLAPACK libraries compiled against openmpi
Requires: %{name}-common = %{version}-%{release}
Requires: openmpi
Provides: blacs-openmpi = %{version}-%{release}

%description openmpi
ScaLAPACK is a library of high-performance linear algebra routines for parallel distributed 
memory machines. ScaLAPACK solves dense and banded linear systems, least squares problems, 
eigenvalue problems, and singular value problems. 

This package provides ScaLAPACK libraries compiled with openmpi.

%package openmpi-devel
Summary: Development libraries for ScaLAPACK (openmpi)
Requires: %{name}-openmpi = %{version}-%{release}
Requires: openmpi-devel
Provides: blacs-openmpi-devel = %{version}-%{release}

%description openmpi-devel
This package provides development libraries for ScaLAPACK, compiled against openmpi.

%package openmpi-static
Summary: Static libraries for ScaLAPACK (openmpi)
Requires: %{name}-openmpi-devel = %{version}-%{release}
Provides: blacs-openmpi-static = %{version}-%{release}

%description openmpi-static
This package provides static libraries for ScaLAPACK, compiled against openmpi.
%endif

%prep
%setup -q -c
%patch3000 -p1 -b .version
%patch3001 -p1 -b .picfix
%patch3002 -p1 -b .fix57

for i in %{?with_mpich:mpich} %{?with_openmpi:openmpi}; do
  cp -a %{name}-%{version} %{name}-%{version}-$i
done

%build
CC="$CC -std=gnu89"
%global build_fflags %(echo %build_fflags -fallow-argument-mismatch| sed 's|-Werror=format-security||g')
%global dobuild() \
cd %{name}-%{version}-$MPI_COMPILER_NAME ; \
%cmake -DBUILD_SHARED_LIBS:BOOL=OFF -DBUILD_STATIC_LIBS:BOOL=ON -DMPI_BASE_DIR=%{_libdir}/$MPI_COMPILER_NAME %{blasflags} ; \
%cmake_build ;\
%cmake -DBUILD_SHARED_LIBS:BOOL=ON -DBUILD_STATIC_LIBS:BOOL=OFF -DMPI_BASE_DIR=%{_libdir}/$MPI_COMPILER_NAME %{blasflags} ; \
%cmake_build ;\
cd ..

%if %{with mpich}
export MPI_COMPILER_NAME=mpich
%{_mpich_load}
%dobuild
%{_mpich_unload}
%endif

%if %{with openmpi}
export MPI_COMPILER_NAME=openmpi
%{_openmpi_load}
%dobuild
%{_openmpi_unload}
%endif

%install
mkdir -p %{buildroot}%{_libdir}/cmake
mkdir -p %{buildroot}%{_bindir}

for i in %{?with_mpich:mpich} %{?with_openmpi:openmpi}; do
  mkdir -p %{buildroot}%{_libdir}/$i/lib/
  pushd %{name}-%{version}-$i
  %cmake_install
  cp -f %{_vpath_builddir}/lib/libscalapack.a %{buildroot}%{_libdir}/$i/lib/
  popd
  mkdir -p %{buildroot}%{_includedir}/$i-%{_arch}/
  # this file is independent of the MPI compiler used, put it in %{_includedir}/blacs/
  mkdir -p %{buildroot}%{_includedir}/blacs/
  install -p %{name}-%{version}-$i/BLACS/SRC/Bdef.h %{buildroot}%{_includedir}/blacs/

  for f in *.so*; do
    mv %{buildroot}%{_libdir}/$f %{buildroot}%{_libdir}/$i/lib/
  done
  mv %{buildroot}%{_libdir}/pkgconfig %{buildroot}%{_libdir}/$i/lib/
done

cd %{name}-%{version}
cp -f README ../

sed -i 's|mpi|ompi|g' %{buildroot}%{_libdir}/openmpi/lib/pkgconfig/scalapack.pc

%files common
%doc README
%{_includedir}/blacs/
%{_libdir}/cmake/%{name}-%{version}/

%if %{with mpich}
%files mpich
%{_libdir}/mpich/lib/libscalapack.so.*

%files mpich-devel
%{_includedir}/mpich-%{_arch}/
%{_libdir}/mpich/lib/libscalapack.so
%{_libdir}/mpich/lib/pkgconfig/scalapack.pc

%files mpich-static
%{_libdir}/mpich/lib/libscalapack.a
%endif

%if %{with openmpi}
%files openmpi
%{_libdir}/openmpi/lib/libscalapack.so.*

%files openmpi-devel
%{_includedir}/openmpi-%{_arch}/
%{_libdir}/openmpi/lib/libscalapack.so
%{_libdir}/openmpi/lib/pkgconfig/scalapack.pc

%files openmpi-static
%{_libdir}/openmpi/lib/libscalapack.a
%endif

%changelog
* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 2.2.0-2
- Rebuilt for OpenCloudOS Stream 23.05

* Wed Apr 26 2023 Wang Guodong <gordonwwang@tencent.com> - 2.2.0-1
- initial build
